/*
 * NAME:
 *  cpuid.h
 *
 * AUTHOR:
 *  Ernest Vogelsinger
 *
 * DESCRIPTION:
 *  Detect processor type and mode
 *  Original INTEL code
 *
 * VERSION:
 *  1.0
 *
 * HISTORY:
 *
 * (c) 1991 E. Vogelsinger
 */

#if !defined(_CPUID_DEFINED)

/* ==== types ============================================================ */

typedef unsigned short PROC_MODE;

/* ==== definitions ====================================================== */

#define PROC_8086       0x0001
#define PROC_80286      0x0002
#define PROC_80386      0x0004
#define PROC_80486SX    0x0008
#define PROC_80486DX    0x0808
#define PROC_8087       0x0100
#define PROC_80287      0x0200
#define PROC_80387      0x0400
#define PROC_80487      0x0800
#define MODE_PROTECTED  0x8000


/* ==== macros =========================================================== */

#define MASK_PROC(t)    ((t) & 0x00FF)
#define MASK_COPROC(t)  ((t) & 0x7F00)
#define IS486DX(t)      (((t)& 0x0808) == 0x0808)
#define IS_PMODE(t)     ((t) & MODE_PROTECTED)

/* ==== prototypes and public values ===================================== */

PROC_MODE far pascal GetCpuId(void);


#define _CPUID_DEFINED
#endif




